<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) exit;

class Inopal extends PaymentModule
{

	private $_html = '';
	private $_postErrors = array();

	public function __construct()
	{
		$this->name = 'inopal';
		$this->tab = 'payments_gateways';
		$this->version = '2.5';
		$this->author = 'http://modulebank.ir';
		$this->currencies = true;
  		$this->currencies_mode = 'checkbox';
  		$this->controllers = array('request', 'verify');
		parent::__construct();
		$this->page = basename(__FILE__, '.php');
		$this->displayName = 'درگاه پرداخت اینوپال';
		$this->description = 'پرداخت از طریق درگاه پرداخت اینوپال';
		$this->confirmUninstall = 'آیا مطمئن هستید می خواهید جزئیات را حذف کنید ؟';
		$this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
		if (!sizeof(Currency::checkPaymentCurrencies($this->id)))
			$this->warning = 'لطفا واحد پولی درگاه را تنظیم نمایید';
		$config = Configuration::getMultiple(array('inopal_terminal', 'inopal_title', 'inopal_rate', ''));
		if (!isset($config['inopal_terminal'],$config['inopal_title'],$config['inopal_rate']))
			$this->warning = 'لطفا اطلاعات ترمینال درگاه را تنظیم نمایید';
	}

	public function install()
	{
		if (!parent::install() or !Configuration::updateValue('inopal_terminal', '') or !Configuration::updateValue('inopal_title', '') or !Configuration::updateValue('inopal_rate', '') or !$this->registerHook('paymentOptions') or !$this->registerHook('paymentReturn'))
		return false;
		return true;
	}

	public function uninstall()
	{
		if (!Configuration::deleteByName('inopal_terminal') or !Configuration::deleteByName('inopal_title') or !Configuration::deleteByName('inopal_rate') or !parent::uninstall())
		return false;
		return true;
	}

	public function displayFormSettings()
	{
		$this->_html .= '<form action="'.$_SERVER['REQUEST_URI'].'" method="post">
			<fieldset>
				<legend>تنظیمات</legend>
				<label> عنوان نمایشی درگاه اینوپال </label>
				<div class="margin-form"><input type="text" size="30" name="inopal_title" id="inopal_title" value="'.Configuration::get('inopal_title').'" /></div>
				<label> ترمینال درگاه اینوپال </label>
				<div class="margin-form"><input type="text" size="30" name="inopal_terminal" id="inopal_terminal" value="'.Configuration::get('inopal_terminal').'" /></div>
				<label> نرخ ارز </label>
				<div class="margin-form"><input type="number" min="0.01" step="0.01" size="30" name="inopal_rate" id="inopal_rate" value="'.(Configuration::get('inopal_rate')?:1).'" /></div>
				<center><input type="submit" name="submit" value="ذخیره" class="button" /></center>
			</fieldset>
		</form>';
	}

	public function displayConf()
	{
		$this->_html .= '<div class="conf confirm">تنظیمات بروز رسانی شد</div>';
	}

	public function displayErrors()
	{
		foreach ($this->_postErrors as $err)
		$this->_html .= '<div class="alert error">'. $err .'</div>';
	}

	public function getContent()
	{
		$this->displayInoPal();
		$this->postprocess();
		$this->displayFormSettings();
		return $this->_html;
	}
	
	private function postprocess()
	{
		if (isset($_POST['submit']))
		{
			if (empty($_POST['inopal_terminal']))
				$this->_postErrors[] = 'ورود ترمینال درگاه اینوپال الزامیست.';
			if (empty($_POST['inopal_title']))
				$this->_postErrors[] = 'ورود نام نمایشی درگاه اینوپال الزامیست.';
			if (empty($_POST['inopal_rate']))
				$this->_postErrors[] = 'ورود نرخ ارز درگاه اینوپال الزامیست.';
			if (!sizeof($this->_postErrors))
			{
				Configuration::updateValue('inopal_terminal', $_POST['inopal_terminal']);
				Configuration::updateValue('inopal_title', $_POST['inopal_title']);
				Configuration::updateValue('inopal_rate', $_POST['inopal_rate']);
				return $this->displayConf();
			}
			return $this->displayErrors();
		}
	}

	private function displayInoPal()
	{
		$this->_html .= '<h2>درگاه پرداخت اینوپال</h2><img src="../modules/inopal/logo.png" style="float:left; margin-right:15px;"><b>این افزونه امکان پرداخت از طریق درگاه پرداخت اینوپال را برای شما فراهم می کند.</b><br /><br />';
	}

	public function execPayment($cart)
	{
		global $cookie, $smarty;
		$amount = $cart->getOrderTotal(true, Cart::BOTH);
		$rate = Configuration::get('inopal_rate')?:1;
		if ($rate > 0) $amount *= $rate;
		$address  = new Address(intval($cart->id_address_invoice));
		require_once(dirname(__FILE__) . '/inopal_payment_helper.class.php');
		$p = new InoPal_Payment_Helper(Configuration::get('inopal_terminal'));
		$r = $p->paymentRequest(array(
				'amount'   => intval(ceil($amount)),
				'order_id' => $cart->id,
				'phone'    => @$address->phone_mobile,
				'callback' => $this->context->link->getModuleLink($this->name, 'verify', array('ResNum' => $cart->id), true)
			));
		if ($r)
		{
			$p->redirect();
		}
		else
		{
			return 'خطا : ' . $p->error;
		}
	}

	public function confirmPayment($amount)
	{
		require_once(dirname(__FILE__) . '/inopal_payment_helper.class.php');
		$p = new InoPal_Payment_Helper(Configuration::get('inopal_terminal'));
		$r = $p->paymentVerify(array_merge($_POST, array('amount' => $amount)));
		if ($r)
		{
			return array(true, $p->txn_id);
		}
		else
		{
			return array(false, 'خطای استعلام پرداخت : '.$p->error);
		}
	}

	public function showMessages($result)
	{
		$this->_postErrors[] = $result;
		$this->displayErrors();
		echo $this->_html;
		return $result;
	}

	public function hookPaymentOptions($params)
	{
		if (!$this->active) return;
		//if (!$this->checkCurrency($params['cart'])) return;
		$cart = $this->context->cart;
		$total = Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH));
		$view = '<section><p class="payment_module"><img src="modules/inopal/logo.png" alt="'.Configuration::get('inopal_title').'" />'.Configuration::get('inopal_title').' <br /></p></section>';
		$newOption = new PaymentOption();
		$newOption->setModuleName($this->name)->setCallToActionText(Configuration::get('inopal_title'))->setAction($this->context->link->getModuleLink($this->name, 'request', array(), true))->setAdditionalInformation($view);
		return array($newOption);
	}

}
